function [V F mappedX]= vca(X,epsilon,varargin)
%VCA   Vanishing Component Analysis. 
%   [V F mappedX] = VCA(X,epsilon) computes a set of polynomials that
%   approximately vanish on the sample set X (the vanishing components). V
%   contains the vanishing components, they are represented by the set of
%   non vanishing components F. mappedX is the the matrix X evaluated on
%   the polynomials in F. epsilon is the level of tolerance.
%   The algorithm continues until the number of elements in F equals the  
%   size of X.
%   [V F mappedX] = VCA(X,epsilon,'degree',d,'features,f,'vanishing',v)
%   will stop when all polynomials of degree d are computed or the size of
%   F is larger then f or the size of V is larger then v.
%
% This file can be obtained from http://www.cs.huji.ac.il/~rlivni73
% You are free to use, change, or redistribute this code in any way you
% want for non-commercial purposes.
%
% (C) Roi Livni, 2013
% Hebrew University of Jerusalem
% see also out_of_sample_vca


%% Create feature set of rank 0
F.nPerRank=1;
V.nPerRank=0;
F.rank{1}=1/sqrt(size(X,1)); %% constant polynom
V.rank{1}=[];
mappedX=[];
criteriaStop=verify_stop(F.nPerRank,V.nPerRank,varargin);

%% Iteratively construct new set of features.
while ~criteriaStop
    [Ft Vt mappedX]= find_range_null(F,X,epsilon,mappedX);
    F.rank{end+1}=Ft; %Features with high variance
    V.rank{end+1}=Vt; %Features with low variance.
    F.nPerRank(end+1)=size(Ft,2);
    V.nPerRank(end+1)=size(Vt,2);
    criteriaStop=verify_stop(F.nPerRank,V.nPerRank,varargin);
end
V.F=F;



function toStop=verify_stop(nF,nV,varargin)
varargin=varargin{1};
toStop=false;
if nF(end)==0
    toStop=true;
    return
end
if ~isempty(varargin)
    for c=1:length(varargin)
        switch varargin{c}
            case('degree')
                if length(nF)>= (varargin{c+1}+1)
                    toStop=true;
                end
            case('features')
                if sum(nF)>= varargin{c+1}
                    toStop=true;
                end
            case('vanishing')
                if sum(nV)>= varargin{c+1}
                    toStop=true;
                end
        end
    end
end




